package example.emr;

import java.io.IOException;
import java.util.Arrays;

import com.amazonaws.auth.PropertiesCredentials;
import com.amazonaws.services.elasticmapreduce.AmazonElasticMapReduce;
import com.amazonaws.services.elasticmapreduce.AmazonElasticMapReduceClient;
import com.amazonaws.services.elasticmapreduce.model.AddJobFlowStepsRequest;
import com.amazonaws.services.elasticmapreduce.model.DescribeJobFlowsRequest;
import com.amazonaws.services.elasticmapreduce.model.DescribeJobFlowsResult;
import com.amazonaws.services.elasticmapreduce.model.HadoopJarStepConfig;
import com.amazonaws.services.elasticmapreduce.model.JobFlowDetail;
import com.amazonaws.services.elasticmapreduce.model.JobFlowInstancesConfig;
import com.amazonaws.services.elasticmapreduce.model.PlacementType;
import com.amazonaws.services.elasticmapreduce.model.RunJobFlowRequest;
import com.amazonaws.services.elasticmapreduce.model.RunJobFlowResult;
import com.amazonaws.services.elasticmapreduce.model.StepConfig;
import com.amazonaws.services.elasticmapreduce.model.StepDetail;
import com.amazonaws.services.elasticmapreduce.model.TerminateJobFlowsRequest;
import com.amazonaws.services.elasticmapreduce.util.StepFactory;

public class EMRManager {
    private static final String AWS_CREDENTIALS_PROPERTIES = "/example/emr/AWSCredentials.properties";

    private static final int pollingSecond = 1 * (1000 * 60);

    private static final String MAP_REDUCE_NAME = "EMR Job";
    private static final String ACTION_ON_FAILURE = "CANCEL_AND_WAIT";
    private static final String ACTION_ON_TERMINATE = "TERMINATE_JOB_FLOW";
    private static final String EMR_DEBUGGIN_NAME = "Debugging";

    private static final String STEP_DETAIL_STATUS_PENDING = "PENDING";
    private static final String STEP_DETAIL_STATUS_RUNNING = "RUNNING";
    private static final String STEP_DETAIL_STATUS_COMPLETED = "COMPLETED";

    private static final String INSTANCE = "m1.large";
    private static final String AVAILABILITY_ZONE = "us-east-1a";
    private static final String HADOOP_VERSION = "0.20";
    private static final String KEY_PAIR_NAME = "cluster-key";
    private static final int INSTANCE_NUMBER = 10;
    private static final String MAPREDUCE_JAR = "s3://example/mr.jar";
    private static final String EMR_LOG_URI = "s3://emr-log/";

    private AmazonElasticMapReduce emr;
    private String jobFlowId;

    public EMRManager() throws IOException {
        emr = new AmazonElasticMapReduceClient(
                new PropertiesCredentials(
                        EMRManager.class.getResourceAsStream(AWS_CREDENTIALS_PROPERTIES)));
    }

    public static void main(String[] args) {
        EMRManager emrManager = null;
        try {
            emrManager = new EMRManager();
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }

        String inputPath1 = "s3://input/1/";
        String outputPath1 = "s3://output/1/";
        emrManager.runJob("wordCount1", inputPath1, outputPath1);

        String inputPath2 = "s3://input/2/";
        String outputPath2 = "s3://output/2/";
        emrManager.runJob("wordCount2", inputPath2, outputPath2);

        emrManager.terminate();

        System.exit(0);
    }

    public void runJob(String jobName, String inputPath, String outputPath) {
        StepConfig stepConfig =
            new StepConfig()
                    .withName("job")
                    .withActionOnFailure(ACTION_ON_FAILURE)
                    .withHadoopJarStep(new HadoopJarStepConfig()
                                                .withJar(MAPREDUCE_JAR)
                                                .withArgs(Arrays.asList(jobName, inputPath, outputPath)));
        if (jobFlowId == null) {
            StepConfig enableDebugging =
                new StepConfig()
                        .withName(EMR_DEBUGGIN_NAME)
                        .withActionOnFailure(ACTION_ON_TERMINATE)
                        .withHadoopJarStep(new StepFactory().newEnableDebuggingStep());

            RunJobFlowRequest runJobFlowRequest =
                new RunJobFlowRequest()
                        .withName(MAP_REDUCE_NAME)
                        .withSteps(enableDebugging, stepConfig)
                        .withLogUri(EMR_LOG_URI)
                        .withInstances(new JobFlowInstancesConfig()
                                                .withEc2KeyName(KEY_PAIR_NAME)
                                                .withHadoopVersion(HADOOP_VERSION)
                                                .withInstanceCount(INSTANCE_NUMBER)
                                                .withKeepJobFlowAliveWhenNoSteps(true)
                                                .withMasterInstanceType(INSTANCE)
                                                .withSlaveInstanceType(INSTANCE)
                                                .withPlacement(new PlacementType()
                                                                    .withAvailabilityZone(AVAILABILITY_ZONE)));

            RunJobFlowResult result = emr.runJobFlow(runJobFlowRequest);
            jobFlowId = result.getJobFlowId();
        } else {
            AddJobFlowStepsRequest addJobFlowStepsRequest =
                new AddJobFlowStepsRequest().withJobFlowId(jobFlowId)
                                            .withSteps(stepConfig);
            emr.addJobFlowSteps(addJobFlowStepsRequest);
        }

        String stepStatus = STEP_DETAIL_STATUS_PENDING;
        while (stepStatus.equals(STEP_DETAIL_STATUS_PENDING) ||
               stepStatus.equals(STEP_DETAIL_STATUS_RUNNING)) {
            try {
                Thread.sleep(pollingSecond);
            } catch (Exception e) {
            }

            DescribeJobFlowsRequest describeJobFlowsRequest =
                new DescribeJobFlowsRequest().withJobFlowIds(jobFlowId);
            DescribeJobFlowsResult describeJobFlowsResult =
                emr.describeJobFlows(describeJobFlowsRequest);
            boolean found = false;
            for (JobFlowDetail jobFlowDetail : describeJobFlowsResult.getJobFlows()) {
                for (StepDetail stepDetail : jobFlowDetail.getSteps()) {
                    if (stepDetail.getStepConfig().getName().equals(jobName)) {
                        stepStatus = stepDetail.getExecutionStatusDetail().getState();
                        found = true;
                        break;
                    }
                }

                if (found) {
                    break;
                }
            }
        }

        if (stepStatus.equals(STEP_DETAIL_STATUS_COMPLETED)) {
            System.out.println("completed");
        } else {
            System.out.println("error");
        }
    }

    /**
     * 
     */
    public void terminate() {
        if (jobFlowId != null) {
            TerminateJobFlowsRequest request = new TerminateJobFlowsRequest();
            request.setJobFlowIds(Arrays.asList(jobFlowId));
            emr.terminateJobFlows(request);
            jobFlowId = null;
        }
    }
}
